/******************************************************************************
* game_mcmc.G
* generates draws from the posterior distribution
* via gibbs sampling
*
* upadted:  06/16/2005 
******************************************************************************/

new;
/** set seed **/
/** rndseed 1234; **/


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/

#include game_spec.g;
#include pathspec.g;
#include game_proc.src;


/********************************************************
* Configure the Gibbs Sampler
********************************************************/

mhrun    = "2";

nblocks = 10;
nsim    = 1000;

/** start log file **/
outstr = resupath $+ "\\mhrun" $+ mhrun $+ "dirsamp.log";
output file = ^outstr reset;

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs00  = n00;          
nobs10  = n10;          
nobs01  = n01;          
nobs11  = n00;          

/********************************************************
*  load dgp parameters
********************************************************/
dgpname = lmodel $+ dataselstr $+ "par.out";
load path = ^priopath thetaalpha_dgp[ndgp_,1] = ^dgpname;

phi_dgp   = (thetaalpha_to_phi(thetaalpha_dgp',msel))';
alpha_dgp = alp12_to_alp(thetaalpha_dgp');

/********************************************************
* Initialize Output files
********************************************************/

opath    = parapath $+ "\\mhrun" $+ mhrun;
chkdir   = chk_dir(opath);
opara    = opath  $+ "\\pa";

/** print specification **/
if _verbose;

    "";
    "===================================";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "MHrun Index      " $mhrun;
    "";
    "===================================";
    "";

    "";
    outmat = ("Parameter"  $~ "True Values") $|
             ( (paranames_) $~ ftocv( (thetaalpha_dgp|phi_dgp|alpha_dgp),4,4) );
    $outmat;

    "";
    "Press Key to Continue";
    k = keyw;
    "";

endif;      /** _verbose **/

/*******************************
** Create new output files
*******************************/

"-----------------------------------------------------------";
"Creating new output files";
"Press Key to Continue";
"-----------------------------------------------------------";
k = keyw;
"";

create fhpara    = ^opara with ^paranames_, (ndgp_+nphi_+nadd_), 4;

/********************************************************
* this algorithm is specified in terms of th1 and phi11, 
* phi00, phi10
********************************************************/

/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   if _verbose;
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);
   endif;

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim   = zeros(nsim,ndgp_+nphi_+nadd_);
   
   /** with-in block loop **/
   j=1;
   do until j > nsim;

      /** sample from marginal posterior of phi **/

      alp_sim = rndgam(4,1,( (n00+1)|(n10+1)|(n01+1)|(n11+1) ) );
      phi_sim = alp_sim./sumc(alp_sim);
      
      /** sample from the conditional prior of theta given phi **/

      th1_min = 0; th1_max = 2.0; th1_n = 101;
      th1_grid = seqa(th1_min,(th1_max-th1_min)/(th1_n-1),th1_n);

      th2_min = 0; th2_max = 2.0; th2_n = 101;
      th2_grid = seqa(th2_min,(th2_max-th2_min)/(th2_n-1),th2_n);

      ids_grid = zeros(th1_n*th2_n,3);

      i1 = 1;
      ith = 1;
      do until i1 > th1_n;

         i2 = 1;
         do until i2 > th2_n;
 
            th1 = th1_grid[i1];
            th2 = th2_grid[i2];

            ids_grid[ith,1:2] = th1~th2;
            ids_grid[ith,3]   = identifiedset12((th1|th2|phi_sim[1:3]),msel);

            i2 = i2+1;
            ith = ith+1;
         endo;

         i1 = i1 + 1;
      endo;

      ids_grid = delif(ids_grid,(ids_grid[.,3].==0));

      /* projection of identified set */

      ids_th1lb = minc(ids_grid[.,1]);
      ids_th1ub = maxc(ids_grid[.,1]);
      ids_th2lb = minc(ids_grid[.,2]);
      ids_th2ub = maxc(ids_grid[.,2]);

      /* use rejection sampler */

      valid = 0;
      do until valid == 1;

        th11_sim = ids_th1lb + rndu(1,1)*(ids_th1ub-ids_th1lb);
        th12_sim = ids_th2lb + rndu(1,1)*(ids_th2ub-ids_th2lb);
        th1_sim  = th11_sim | th12_sim;

        valid = identifiedset12((th1_sim|phi_sim[1:3]),msel); 

      endo;

      
      th2alp_sim   = th1phi_to_th2alp(th1_sim|phi_sim[1:3]);
      parasim[j,.] = th1_sim'~th2alp_sim' ~ 0 ~ phi_sim' ~ th2alp_sim[3];
      
      j = j+1;
   
   endo;               /** with-in block  **/

   if _verbose;
       " ";
       "Theta   "; meanc(parasim[.,1:ntheta_])';
       "True    "; thetaalpha_dgp[1:ntheta_]';
       "" 
       "Alpha   "; meanc(parasim[.,ndgp_+nphi_+1:ndgp_+nphi_+nadd_]);
       "True    "; alpha_dgp;
       "";
       "Phi     "; meanc(parasim[.,ndgp_+1:ndgp_+nphi_])';
       "True    "; phi_dgp[1:nphi_]';       
       "";
       
       pause(1e-20);
   endif;  /** verbose **/

   /** set finish time **/
   tfin = date;

   if _verbose;
       "Elapsed Time"; etstr(ethsec(tstart,tfin));
       "";
       "-----------------------------------------";
       "";
   endif;  /** verbose **/

   /** save output **/
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;  /** block **/

closeall fhpara;

/** close log file **/
output file = ^outstr off;

end;

/**********************************************/

proc (1) = th1phi_to_th2alp(th1phi);
local bet1, bet2, del1, del2, alp,
      phi00, phi01, phi10, phi11,
      m1, m2, d1, d2;

bet1  = th1phi[1];
del1  = th1phi[2];

phi00 = th1phi[3];
phi10 = th1phi[4];
phi01 = th1phi[5];
phi11 = 1-phi00-phi10-phi01;

m1 = cdfn(bet1);
d1 = cdfn(bet1-del1);
m2 = 1 - phi00/(1-m1);
d2 = phi11/d1; 

alp = ( phi10 - m1*(1-m2) - d1*(m2-d2) )/( (m1-d1)*(m2-d2));
bet2 = cdfni(m2);
del2 = bet2 - cdfni(d2);

retp(bet2|del2|alp);
endp;

/**********************************************/

proc (1) = identifiedset12(th1phi,msel);
/* Check whether th1 and th2 are part of the identified set
*/
local bet1, del1, bet2, del2,
      phi00, phi10, phi01, phi11,
      m1, d1, m2, d2, alp,
      idset;

phi00 = th1phi[3];
phi10 = th1phi[4];
phi01 = th1phi[5];
phi11 = 1-phi00-phi10-phi01;

bet1  = th1phi[1];
del1  = th1phi[2];

/* compute monopoly and duopoly profits for firm 1
** based on structural parameters
*/
m1 = cdfn(bet1);
d1 = cdfn(bet1-del1);
idset = 0;

/* compute monopoly and duopoly probabilities for firm 2
** based on reduced form and firm 1 structural parameters
*/
m2 = 1 - phi00/(1-m1);
d2 = phi11/d1; 
      
if (m2 ge 0) and (m2 le 1) and (d2 ge 0) and (d2 le 1) and (m2 ge d2);
       alp = ( phi10 - m1*(1-m2) - d1*(m2-d2) )/( (m1-d1)*(m2-d2)) ;
       if (alp ge 0) and (alp le 1);
               idset = 1;
        endif;
endif;
                      
retp(idset);
endp;