/******************************************************************************
* game_freqobj.G
* compute a frequentist confidence set 
*
* updated:  06/16/2005 
******************************************************************************/

new;
/** set seed **/
/** rndseed 1234; **/

/* Global variables are denoted by X_
** msel_
** th12_
** phihat_
** lambda_hat_
** nobsall_
*/
 
_sqp_FeasibleTest = 0;
_sqp_PrintIters   = 0;
__output          = 0;
_sqp_MaxIters     = 2000;

_sqp_Bounds = (- 10 ~ 10 ) |
              (  0 ~  10 ) ;

/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/

#include game_spec.g;
#include pathspec.g;
#include game_proc.src;


/********************************************************
* Configuration
********************************************************/

freqindx= 1;
critval_ = cdfchii(0.6,3);


/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobsall = n00+n10+n01+n11;
phi_hat = n11/nobsall |
          n00/nobsall |
          n10/nobsall ;

n_rep = 1E5;          
sd = 1234;
lambda_hat = lambdaproc(phi_hat,nobsall,n_rep,sd);

/* set globals */
nobsall_    = nobsall;
msel_       = msel;
lambda_hat_ = lambda_hat;
phi_hat_    = phi_hat;
th34true    = 0.5 | 1;

/********************************************************
*  Create a grid for the th1 and th2
*  We compute a CI for th1 and th2 using a projection method
********************************************************/

if freqindx == 1;
   /*
   th1_min = 0; th1_max = 2; th1_n = 51;
   th1_grid = seqa(th1_min,(th1_max-th1_min)/(th1_n-1),th1_n);

   th2_min = 0; th2_max = 2.0; th2_n = 51;
   th2_grid = seqa(th2_min,(th2_max-th2_min)/(th2_n-1),th2_n);
   */

th1_min = 0; th1_max = 2; th1_n = 201;
th1_grid = seqa(th1_min,(th1_max-th1_min)/(th1_n-1),th1_n);

th2_min = 0; th2_max = 2; th2_n = 201;
th2_grid = seqa(th2_min,(th2_max-th2_min)/(th2_n-1),th2_n);

endif;

/********************************************************
*  Initialize Output Matrices
********************************************************/

/* vectorized output 
** first two columns are grid values
*/
QTh_vec    = zeros(th1_n*th2_n,3);
IDS12_vec  = zeros(th1_n*th2_n,3);

/* output in matrix form 
** right-most and bottom column contain grid values
*/
Qth_mat                  = zeros( (th2_n+1), (th1_n+1) );
Qth_mat[1:th2_n,th1_n+1] = th2_grid;
Qth_mat[th2_n+1,1:th1_n] = th1_grid';


IDS12_mat  = zeros( (th2_n+1), (th1_n+1) );
IDS12_mat[1:th2_n,th1_n+1] = th2_grid;
IDS12_mat[th2_n+1,1:th1_n] = th1_grid';

/********************************************************
* Initialize Output files
********************************************************/

opath    = resupath $+ "\\freq" $+ ftos(freqindx,"%*.*lf",1,0);
chkdir   = chk_dir(opath);
oqthvec  = opath  $+ "\\Qthvec";
oqthmat  = opath  $+ "\\Qthmat";
oids12vec= opath  $+ "\\IDS12vec";
oids12mat= opath  $+ "\\IDS12mat";

/** start log file **/
outstr = resupath $+ "\\freq" $+ ftos(freqindx,"%*.*lf",1,0) 
         $+ ftos(timedt,"%*.*lf",14,0) $+ "freqcs.log";
output file = ^outstr reset;

/********************************************************
*  Compute the Objective Function
********************************************************/

ict = 1;
i1  = 1;
do until i1 > th1_n;

   i2 = 1;
   do until i2 > th2_n;

       th12_ = th1_grid[i1] |
               th2_grid[i2] ;
    
       /* compute alternative starting values
       */
       m1 = cdfn(th12_[1]);
       d1 = cdfn(th12_[1]-th12_[2]);
       d2 = phi_hat[1]/d1;
       m2 = 1 - phi_hat[2]/(1-m1);
   
       if (m2 ge 0) and (m2 le 1) and (d2 ge 0) and (d2 le 1) and (m2 ge d2);
         bet2 = cdfni(m2);
         del2 = bet2 - cdfni(d2);
         th34start_ = (bet2|del2);
       else;
         th34start_ = th34true;
       endif;
       
       /* use minimization to get rid of th34*/
       { th34_hat, Q_hat, lagr, ret } = sqpSolve( &qobjfcn,th34start_ );     
       ids12 = identifiedset12(th12_,phi_hat,msel);  
       if ids12 == 1;
          Q_hat ids12;
          pause(1e-5);
       endif;
                   
       QTh_vec[ict,1]   = Q_hat;
       QTh_vec[ict,2:3] = th12_';
       QTh_mat[i2,i1]   = Q_hat;
       
       IDS12_vec[ict,1]   = ids12;
       IDS12_vec[ict,2:3] = th12_';
       IDS12_mat[i2,i1]   = ids12;

       if fmod(ict,rows(Qth_vec)/10)==0; 
          /* ".";; */
          "Step" ict "out of" rows(Qth_vec); 
          pause(1e-5); 
       endif;
              
       i2 = i2 + 1;
       ict = ict+1;
       
   endo;
   
   i1 = i1+1;
   
endo;

/********************************************************
*  Write Output to Disk
********************************************************/


create fhqthvec = ^oqthvec with QTHvec, cols(Qth_vec), 8;
wr = writer(fhqthvec,Qth_vec);
closeall fhqthvec;

create fhqthmat = ^oqthmat with QTHmat, cols(Qth_mat), 8;
wr = writer(fhqthmat,Qth_mat);
closeall fhqthmat;

create fhids12vec = ^oids12vec with ids12vec, cols(IDS12_vec), 8;
wr = writer(fhids12vec,IDS12_vec);
closeall fhids12vec;

create fhids12mat = ^oids12mat with ids12mat, cols(IDS12_mat), 8;
wr = writer(fhids12mat,IDS12_mat);
closeall fhids12mat;


end;

/********************************************************
* freqobjfcn
********************************************************/

proc (1) = qobjfcn(th34);

local phi11_hat, phi00_hat, phi10_hat,
      LA11, LA22, LA21, LA211, 
      bet1, del1, bet2, del2,
      m1, m2, d1, d2,
      G00, G11, G10, G1001,
      h00_hat, h11_hat, h10_hat,
      HH1_hat, HH2_hat, HH211_hat,
      Qobj; 
 
/* reduced form parameter estimates
*/
phi11_hat = phi_hat_[1];
phi00_hat = phi_hat_[2];
phi10_hat = phi_hat_[3];


/* partition lambda
*/
LA11 = lambda_hat_[1:2,1:2];
LA22 = lambda_hat_[3,3];
LA21 = lambda_hat_[3,1:2];
LA211 = LA22 - LA21*inv(LA11)*LA21';

/* bounds for reduced-form parameters
*/

bet1  = th12_[1];
del1  = th12_[2];
bet2  = th34[1];
if msel[1] == 1;
    del2  = th34[2];
elseif msel[1] == 2;
    del2  = del1;
endif;

m1 = cdfn(bet1);
m2 = cdfn(bet2);
d1 = cdfn(bet1-del1);
d2 = cdfn(bet2-del2);

G11   = d1*d2;
G00   = (1-m1)*(1-m2);
G10   = m1*(1-d2);
G1001 = (m1-d1)*(m2-d2);

/* h functions 
*/
h11_hat = phi11_hat - G11;
h00_hat = phi00_hat - G00;
h10_hat = phi10_hat - G10;

HH1_hat = h11_hat | h00_hat;
HH2_hat = h10_hat;
HH211_hat = HH2_hat - LA21*inv(LA11)*HH1_hat;

/* compute objective function
*/

if HH211_hat ge 0;
   /* CASE 1 */
   Qobj = nobsall_*HH1_hat'*inv(LA11)*HH1_hat + nobsall_*HH211_hat'*inv(LA211)*HH211_hat;
elseif (-G1001 < HH211_hat) and (HH211_hat < 0);
   /* CASE 2 */
   Qobj = nobsall_*HH1_hat'*inv(LA11)*HH1_hat;
else;
   /* CASE 3 */
   Qobj = nobsall_*HH1_hat'*inv(LA11)*HH1_hat + nobsall_*(HH211_hat+G1001)'*inv(LA211)*(HH211_hat+G1001);
endif;   

if Qobj < 0.95*critval_;
   Qobj = 0.95*critval_;
endif;

retp(Qobj);
endp;

/********************************************************
* Identified Set for th1 and th2
********************************************************/

proc (1) = identifiedset12(theta,phi,msel);
/* Check whether th1 and th2 are part of the identified set
*/
local bet1, del1, bet2, del2,
      phi00, phi10, phi01, phi11,
      m1, d1, m2, d2, alp,
      idset;

phi11 = phi[1];
phi00 = phi[2];
phi10 = phi[3];
phi01 = 1-phi11-phi00-phi10;

bet1  = theta[1];
del1  = theta[2];

/* compute monopoly and duopoly profits for firm 1
** based on structural parameters
*/
m1 = cdfn(bet1);
d1 = cdfn(bet1-del1);
idset = 0;

/* compute monopoly and duopoly probabilities for firm 2
** based on reduced form and firm 1 structural parameters
*/
m2 = 1 - phi00/(1-m1);
d2 = phi11/d1; 
      
if (m2 ge 0) and (m2 le 1) and (d2 ge 0) and (d2 le 1) and (m2 ge d2);
            alp = ( phi10 - m1*(1-m2) - d1*(m2-d2) )/( (m1-d1)*(m2-d2)) ;
            if (alp ge 0) and (alp le 1);
               idset = 1;
            endif;
endif;
                      
retp(idset);
endp;

/********************************************************
* lambda_hat
********************************************************/

proc (1) = lambdaproc(phi_hat,n_samp,n_rep,sd);
local phi_mat, phi_m, ind, YY, lambda_hat;

phi_mat = zeros(n_rep,3);

rndseed sd;

ind = 1;
do until ind > n_rep;

   /* create a sample of size n_samp */
   YY = rndu(n_samp,1);
   phi_mat[ind,1] = meanc( YY .< phi_hat[1] );
   phi_mat[ind,2] = meanc( (YY .GE phi_hat[1]) .AND (YY .< (phi_hat[1]+phi_hat[2])) );
   phi_mat[ind,3] = meanc( (YY .GE (phi_hat[1]+phi_hat[2])) .AND (YY .< (phi_hat[1]+phi_hat[2]+phi_hat[3])) ); 
   ind = ind+1;
   
endo;

phi_m = meanc(phi_mat);
lambda_hat = n_samp*(phi_mat'*phi_mat/n_rep - phi_m*phi_m');

retp(lambda_hat);
endp;
