/* filename:    hpdset2dim.g
** description: Computes HPD set via kernel density set 
**              based on the output of the posterior simulator
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

phiindx = 1;

mhrun  = "1prior" $+ ftos(phiindx,"%*.*lf",1,0);    

nburn   = 101;        /* Number of initial draws to be discarded */
nuse    = 100;
nblock  = 100000;
covprob = 0.60;

#include game_spec.g;
#include game_proc.src;
#include pathspec.g;

/********************************************************
* Set Reduced form parameters
********************************************************/

/* either load parameters of DGP
** or specify theta-alpha parameters directly
*/
dgpname   = lmodel $+ dataselstr $+ "par.out";
load path = ^priopath thetaalpha[ndgp_,1] = ^dgpname;

if phiindx == 1;
   /* true values */
   phi = (thetaalpha_to_phi(thetaalpha',msel))';
elseif phiindx == 2;
   /* some other values */
   thetaalpha_alt = 1.3 | 1.2 | 0.4 | 1.2 | 0.3 | 0;
   phi = (thetaalpha_to_phi(thetaalpha_alt',msel))';
endif;
/* reorder phi vector */
phi = phi[4 1 2];

/********************************************************
*  Create a grid for the th1 and th2
********************************************************/

th1_min = 0; th1_max = 2; th1_n = 101;
th1_grid = seqa(th1_min,(th1_max-th1_min)/(th1_n-1),th1_n);

th2_min = 0; th2_max = 2; th2_n = 101;
th2_grid = seqa(th2_min,(th2_max-th2_min)/(th2_n-1),th2_n);


/********************************************************
*  Load first parameter draws (for density estimate)
********************************************************/

"Loading Draws for Density Estimates";
pause(1e-5);

lpath  = parapath $+ "\\mhrun1prior" $+ ftos(phiindx,"%*.*lf",1,0);
ldraw  = lpath $+ "\\pa";

open fhdraw = ^ldraw for read;

/** Initialization **/
drawrow = seekr( fhdraw, nburn);
eofloop = 0;

drawvec1 = readr( fhdraw,1);
drawvec1 = drawvec1[.,1:2];

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr( fhdraw, nblock );
   nread   = rows( parasim );

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

   if ismiss(parasim) == 0;
      drawvec1 = drawvec1 | parasim[.,1:2];
   endif;

   eofloop = eof(fhdraw);

endo;
closeall fhdraw;

drawstdd1 = stdc(drawvec1);
/*h = 0.2*rows(drawvec1)^(-1/6)*drawstdd1;*/
h = rows(drawvec1)^(-1/6)*drawstdd1;

/********************************************************
*  load second parameter draws (to evaluate density)
********************************************************/

"Evaluating Density";
pause(1e-5);

lpath  = parapath $+ "\\mhrun2prior" $+ ftos(phiindx,"%*.*lf",1,0);
ldraw  = lpath $+ "\\pa";

open fhdraw = ^ldraw for read;

/** Initialization **/
drawrow = seekr( fhdraw, nburn);
eofloop = 0;
loopct  = 0;

densvec = 0;

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr( fhdraw, nblock );
   nread   = rows( parasim );

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

   loopct = loopct + nread;

   if ismiss(parasim) == 0;

      densvec = densvec | densest(parasim[.,1:2],drawvec1,h);
      "Draws processed:" loopct;
      pause(1e-5);   

   endif;

   eofloop = eof(fhdraw);

endo;
closeall fhdraw;

densvec     = sortc(densvec[2:rows(densvec)],1);
covprobindx = round( (1-covprob)*rows(densvec) );
hpdcutoff   = densvec[covprobindx];

/********************************************************
*  construct hpd density set
********************************************************/

/* output in matrix form 
** right-most and bottom column contain grid values
*/
hpd_mat                  = zeros( (th2_n+1), (th1_n+1) );
hpd_mat[1:th2_n,th1_n+1] = th2_grid;
hpd_mat[th2_n+1,1:th1_n] = th1_grid';


IDS12_mat  = zeros( (th2_n+1), (th1_n+1) );
IDS12_mat[1:th2_n,th1_n+1] = th2_grid;
IDS12_mat[th2_n+1,1:th1_n] = th1_grid';

/* Evaluate density on grid
** and determine whether parameter belongs to HPD set 
*/

"Determining HPD Set";
pause(1e-5);

ict = 1;
i1  = 1;
do until i1 > th1_n;

   i2 = 1;
   do until i2 > th2_n;

      th12 = th1_grid[i1] ~ th2_grid[i2] ;
      IDS12_mat[i2,i1] = identifiedset12(th12,phi);  
      if IDS12_mat[i2,i1] > 0;
         hpd_mat[i2,i1] = ( densest(th12,drawvec1,h) .ge hpdcutoff ) ;
      endif;

      if fmod(ict,th1_n*th2_n/10)==0; 
          /* ".";; */
          "Step" ict "out of" th1_n*th2_n; 
          pause(1e-20); 
      endif;
              
      i2 = i2 + 1;
      ict = ict+1;
       
   endo;
   
   i1 = i1+1;
   
endo;

/********************************************************
* Derive boundaries of HPD set
********************************************************/

HPDnogrid_mat = HPD_mat[1:th2_n,1:th1_n];

HPD_matsel = ( sumc(HPDnogrid_mat) .== 0 ); 
HPDtr_mat  = ( delif(HPDnogrid_mat',HPD_matsel) )';
HPDth1_grid   = ( delif(th1_grid,HPD_matsel) );

HPD_mat_temp = seqa(1,1,th2_n) .* HPDtr_mat;
HPD_ub = th2_grid[maxindc(HPD_mat_temp[1:th2_n,.])'];

HPD_mat_temp = seqa(th2_n,-1,th2_n) .* HPDtr_mat;
HPD_lb = th2_grid[maxindc(HPD_mat_temp[1:th2_n,.])'];

/* check whether there are open ends */

if HPD_ub[1] ne HPD_lb[1];
   HPDth1_grid = (HPDth1_grid[1] - (HPDth1_grid[2]-HPDth1_grid[1])/100) |
                 HPDth1_grid;
   HPD_diff = (HPD_ub[1]-HPD_lb[1])/2;
   HPD_ub   = (HPD_lb[1]+HPD_diff) | HPD_ub;
   HPD_lb   = (HPD_lb[1]+HPD_diff) | HPD_lb;
endif;

HPDth1_n = rows(HPDth1_grid);

if HPD_ub[HPDth1_n] ne HPD_lb[HPDth1_n];
   HPDth1_grid = HPDth1_grid |
                (HPDth1_grid[HPDth1_n] + (HPDth1_grid[HPDth1_n]-HPDth1_grid[HPDth1_n-1])/100);
   HPD_diff = (HPD_ub[HPDth1_n]-HPD_lb[HPDth1_n])/2;
   HPD_ub   = HPD_ub | (HPD_lb[HPDth1_n]+HPD_diff);
   HPD_lb   = HPD_lb | (HPD_lb[HPDth1_n]+HPD_diff);
endif;

/********************************************************
* Derive boundaries of identified set
********************************************************/

IDS12nogrid_mat = IDS12_mat[1:th2_n,1:th1_n];

IDS12_matsel = ( sumc(IDS12nogrid_mat) .== 0 ); 
IDS12tr_mat  = ( delif(IDS12nogrid_mat',IDS12_matsel) )';
IDS12th1_grid   = ( delif(th1_grid,IDS12_matsel) );

IDS12_mat_temp = seqa(1,1,th2_n) .* IDS12tr_mat;
IDS12_ub = th2_grid[maxindc(IDS12_mat_temp[1:th2_n,.])'];

IDS12_mat_temp = seqa(th2_n,-1,th2_n) .* IDS12tr_mat;
IDS12_lb = th2_grid[maxindc(IDS12_mat_temp[1:th2_n,.])'];

/* close open ends */
if IDS12_ub[1] ne IDS12_lb[1];
   IDS12th1_grid = (IDS12th1_grid[1] - (IDS12th1_grid[2]-IDS12th1_grid[1])/100) |
                 IDS12th1_grid;
   IDS12_diff = (IDS12_ub[1]-IDS12_lb[1])/2;
   IDS12_ub   = (IDS12_lb[1]+IDS12_diff) | IDS12_ub;
   IDS12_lb   = (IDS12_lb[1]+IDS12_diff) | IDS12_lb;
endif;

IDS12th1_n = rows(IDS12th1_grid);

if IDS12_ub[IDS12th1_n] ne IDS12_lb[IDS12th1_n];
   IDS12th1_grid = IDS12th1_grid |
                (IDS12th1_grid[IDS12th1_n] + (IDS12th1_grid[IDS12th1_n]-IDS12th1_grid[IDS12th1_n-1])/100);
   IDS12_diff = (IDS12_ub[IDS12th1_n]-IDS12_lb[IDS12th1_n])/2;
   IDS12_ub   = IDS12_ub | (IDS12_lb[IDS12th1_n]+IDS12_diff);
   IDS12_lb   = IDS12_lb | (IDS12_lb[IDS12th1_n]+IDS12_diff);
endif;

/********************************************************
* Initialize Output files
********************************************************/

opath    = resupath $+ "\\mhrun" $+ mhrun;
chkdir   = chk_dir(opath);
ohpd     = opath  $+ "\\HPD2dim" $+ ftos(100*covprob,"%*.*lf",2,0);

create fhhpd = ^ohpd with HPD, rows(HPDth1_grid), 8;
wr = writer(fhhpd,(HPDth1_grid~HPD_ub~HPD_lb)');
closeall fhhpd;

oids     = opath  $+ "\\IDS2dim";
create fhids = ^oids with IDS, rows(IDS12th1_grid), 8;
wr = writer(fhids,(IDS12th1_grid~IDS12_ub~IDS12_lb)');
closeall fhids;


/********************************************************
* Plot some results
********************************************************/

graphset;
begwind;
margin(0.2,0.2,0.2,0.2); 

window(1,1,0);
fonts("microb");
_pcolor = {2 2};
_pltype = {6 6};
_plwidth = 8;
_psymsiz = 3;
_plctrl = -1;
_plctrl=0;

xtics(0,2,0.5,1);
ytics(0,2,0.5,1);

xy(HPDth1_grid,HPD_ub~HPD_lb);

_pcolor = {4 4};
_pltype = {2 2};

xy(IDS12th1_grid,IDS12_ub~IDS12_lb);

endwind;

end;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/


proc (1) = densest(paragrid,draws,h);
/* This procedure computes a 2dim kernel density estimate
** using indep Gaussian kernels
*/
local   ndraws, npt, pdfhat, i;

ndraws = rows(draws);
npt    = rows(paragrid);
pdfhat = zeros(npt,1); 

i = 1;
do until i > npt;
   pdfhat[i] = 1/(h[1]*h[2])
               *meanc(   pdfn( (ones(ndraws,1)*paragrid[i,1]-draws[.,1])/h[1] )
                     .* pdfn( (ones(ndraws,1)*paragrid[i,2]-draws[.,2])/h[2] ) );
   i = i+1;
endo;

retp(pdfhat);
endp;

proc (1) = identifiedset12(theta,phi);
/* Check whether th1 and th2 are part of the identified set
*/
local bet1, del1, bet2, del2,
      phi00, phi10, phi01, phi11,
      m1, d1, m2, d2, alp,
      idset;

phi11 = phi[1];
phi00 = phi[2];
phi10 = phi[3];
phi01 = 1-phi11-phi00-phi10;

bet1  = theta[1];
del1  = theta[2];

/* compute monopoly and duopoly profits for firm 1
** based on structural parameters
*/
m1 = cdfn(bet1);
d1 = cdfn(bet1-del1);
idset = 0;

/* compute monopoly and duopoly probabilities for firm 2
** based on reduced form and firm 1 structural parameters
*/
m2 = 1 - phi00/(1-m1);
d2 = phi11/d1; 
      
if (m2 ge 0) and (m2 le 1) and (d2 ge 0) and (d2 le 1) and (m2 ge d2);
            alp = ( phi10 - m1*(1-m2) - d1*(m2-d2) )/( (m1-d1)*(m2-d2)) ;
            if (alp ge 0) and (alp le 1);
               idset = 1;
            endif;
endif;
         
             
retp(idset);
endp;