/******************************************************************************
* game_mcmc.G
* generates draws from the posterior distribution
* via gibbs sampling
*
* upadted:  06/16/2005 
******************************************************************************/

new;
/** set seed **/
/** rndseed 1234; **/


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/

#include game_spec.g;
#include pathspec.g;
#include game_proc.src;


/********************************************************
* Configure the Gibbs Sampler
********************************************************/

mhrun    = "1";

nblocks = 10;
nsim    = 100000;

/** start log file **/
outstr = resupath $+ "\\mhrun" $+ mhrun 
         $+ ftos(timedt,"%*.*lf",14,0) $+ "rwm.log";
output file = ^outstr reset;

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs00  = n00;          
nobs10  = n10;          
nobs01  = n01;          
nobs11  = n00;          

phi_hat = (n00|n10|n01|n11)/(n00+n10+n01+n11);

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/********************************************************
*  load dgp parameters
********************************************************/
dgpname = lmodel $+ dataselstr $+ "par.out";
load path = ^priopath thetaalpha_dgp[ndgp_,1] = ^dgpname;

phi_dgp   = (thetaalpha_to_phi(thetaalpha_dgp',msel))';
alpha_dgp = alp12_to_alp(thetaalpha_dgp');

/********************************************************
**  Covariance matrix for proposal density
**  Initialization
**  cc = 0.3 for 50 obs
**  cc = 0.2 for 200 obs
**  cc = 0.1  for 1,000 obs
**  cc = 0.05 for 10,000 obs
********************************************************/
cc = 0.1;
V_prop = cc*diagrv( eye(ndgp_), (1|1|1|1|0.25|0.25) );

/* set entries corresponding to fixed parameters equal to zero
*/
V_prop = V_prop .* (pmaskinv*pmaskinv');

/* Create Cholesky factor
*/
V_propchol = cholpsd(V_prop);

/********************************************************
* Initialize Output files
********************************************************/

opath    = parapath $+ "\\mhrun" $+ mhrun;
chkdir   = chk_dir(opath);
opara    = opath  $+ "\\pa";

/** print specification **/
if _verbose;

    "";
    "===================================";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "MHrun Index      " $mhrun;
    "";
    "===================================";
    "";

    "";
    outmat = ("Parameter"  $~ "True Values") $|
             ( (paranames_) $~ ftocv( (thetaalpha_dgp|phi_dgp|alpha_dgp),4,4) );
    $outmat;

    "";
    "Press Key to Continue";
    k = keyw;
    "";

endif;      /** _verbose **/

/*******************************
** Create new output files
*******************************/

"-----------------------------------------------------------";
"Creating new output files";
"Press Key to Continue";
"-----------------------------------------------------------";
k = keyw;
"";

create fhpara    = ^opara with ^paranames_, (ndgp_+nphi_+nadd_), 4;

/********************************************************
* RWM
********************************************************/

/** starting values **/
thetaalpha_sim = thetaalpha_dgp;
lnpost_sim = lnpostdens( (n00|n10|n01|n11),thetaalpha_sim,pmean,pstdd,pshape,msel);

/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   if _verbose;
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);
   endif;

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim   = zeros(nsim,ndgp_+nphi_+nadd_);
   rej = zeros(nsim,1);
   
   /** with-in block loop **/
   j=1;
   do until j > nsim;

      /** propose **/
      thetaalpha_new = thetaalpha_sim + cc*(V_propchol*rndn(ndgp_,1));
      
      /** evaluate at the candidate **/
      lnpost_new = lnpostdens( (n00|n10|n01|n11),thetaalpha_new,pmean,pstdd,pshape,msel);
      
      /** accept/reject rule  **/
      r = minc(1 | exp( lnpost_new - lnpost_sim));

      if rndu(1,1) < r;

          /** accept proposed jump **/
          thetaalpha_sim = thetaalpha_new;
          lnpost_sim = lnpost_new;

      else;

          rej[j] = 1;

      endif;
      
      phi_sim = thetaalpha_to_phi(thetaalpha_sim',msel);
      alp_sim = alp12_to_alp(thetaalpha_sim');
      parasim[j,.] = thetaalpha_sim' ~ phi_sim ~ alp_sim;
      
      j = j+1;
   
   endo;               /** with-in block  **/

   if _verbose;
       " ";
       "Rejection Rate" meanc(rej);
       " ";
       "Theta   "; meanc(parasim[.,1:ntheta_])';
       "True    "; thetaalpha_dgp[1:ntheta_]';
       "" 
       "Phi     "; meanc(parasim[.,ndgp_+1:ndgp_+nphi_])';
       "Phi-hat "; phi_hat';
       "True    "; phi_dgp[1:nphi_]';       
       "";
       
       pause(1e-20);
   endif;  /** verbose **/

   /** set finish time **/
   tfin = date;

   if _verbose;
       "Elapsed Time"; etstr(ethsec(tstart,tfin));
       "";
       "-----------------------------------------";
       "";
   endif;  /** verbose **/

   /** save output **/
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;  /** block **/

closeall fhpara;

/** close log file **/
output file = ^outstr off;

end;

/********************************************************
* posterior density
********************************************************/

proc (1) = lnpostdens(nvec,thetaalpha,pmean,pstdd,pshape,msel);

local bet1, del1, bet2, del2, alp1, alp2, alp,
      phi, lnlike, lnprior, i, a, b;

bet1 = thetaalpha[1];
del1 = thetaalpha[2];
bet2 = thetaalpha[3];
if msel[1] == 1;
   del2 = thetaalpha[4];
else;
   del2 = del1;
endif;
alp1 = thetaalpha[5];
alp2 = thetaalpha[6];

if (alp1 ge 0) and (alp1 le 1)
    and (del1 ge 0) and (del2 ge 0) ;
    /* make sure that alpha is in the unit interval 
    ** and delta is non-negative */
    phi   = thetaalpha_to_phi(thetaalpha',msel);
    lnlike = ln(phi)*nvec;
else;
    lnlike = -1E100;
    lnprior = 0;
    goto eoflnpost;
endif;

lnprior = 0;

i = 1;
do until i > ndgp_;
   if pshape[i] == 1; /* BETA Prior */
      a = (1-pmean[i])*pmean[i]^2/pstdd[i]^2 - pmean[i];
      b = a*(1/pmean[i] - 1);
      lnprior = lnprior + lpdfbeta(thetaalpha[i],a,b);   
   elseif pshape[i] == 2; /* GAMMA PRIOR */
      b = pstdd[i]^2/pmean[i];
      a = pmean[i]/b;
      lnprior = lnprior + lpdfgam(thetaalpha[i],a,b);
   elseif pshape[i] == 3; /* GAUSSIAN PRIOR */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + lpdfnor(thetaalpha[i],a,b);
   elseif pshape[i] == 4; /* INVGAMMA PRIOR */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + lpdfig(thetaalpha[i],a,b);
   elseif pshape[i] == 5; /* Uniform Prior */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + ln(1/(b-a));
   elseif pshape[i] == 6; /* Truncated Normal / symmetric around zero */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + ln(2) + lpdfnor(thetaalpha[i],a,b);
   endif;
   i = i+1;
endo;

eoflnpost:

retp(lnlike+lnprior);
endp;
