/******************************************************************************
* game_mcmc.G
* generates draws from the posterior distribution
* via gibbs sampling
*
* upadted:  06/16/2005 
******************************************************************************/

new;
/** set seed **/
/** rndseed 1234; **/


/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
_verbose = 1;       /** 1 for verbose **/

#include game_spec.g;
#include pathspec.g;
#include game_proc.src;

phiindx = 1;

/********************************************************
* Configure the RWM Sampler
********************************************************/

mhrun    = "1prior" $+ ftos(phiindx,"%*.*lf",1,0);

nblocks = 10;
nsim    = 100000;

/** start log file **/
outstr = resupath $+ "\\mhrun" $+ mhrun  
         $+ ftos(timedt,"%*.*lf",14,0) $+ "rwmcondprior.log";
output file = ^outstr reset;

/********************************************************
* Set Reduced form parameters
********************************************************/

/* either load parameters of DGP
** or specify theta-alpha parameters directly
*/
dgpname   = lmodel $+ dataselstr $+ "par.out";
load path = ^priopath thetaalpha[ndgp_,1] = ^dgpname;

if phiindx == 1;
   /* true values */
   phi = (thetaalpha_to_phi(thetaalpha',msel))';
elseif phiindx == 2;
   /* some other values */
   thetaalpha = 1.3 | 1.2 | 0.4 | 1.2 | 0.3 | 0;
   phi = (thetaalpha_to_phi(thetaalpha',msel))';
endif;

/* reorder phi vector */
phi = phi[4 1 2];



/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
  /* make sure that del2 is fixed if msel[1]=2 */
  if msel[1] == 2; pmask[4]=1; endif;
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/********************************************************
**  Covariance matrix for proposal density
**  Initialization
********************************************************/
cc         = 0.25;
V_prop     = cc*eye(2);
V_propchol = cholpsd(V_prop);

/********************************************************
* Initialize Output files
********************************************************/

opath    = parapath $+ "\\mhrun" $+ mhrun;
chkdir   = chk_dir(opath);
opara    = opath  $+ "\\pa";

/** print specification **/
if _verbose;

    "";
    "===================================";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "MHrun Index      " $mhrun;
    "";
    "===================================";
    "";

    "Press Key to Continue";
    k = keyw;
    "";

endif;      /** _verbose **/

/*******************************
** Create new output files
*******************************/

"-----------------------------------------------------------";
"Creating new output files";
"Press Key to Continue";
"-----------------------------------------------------------";
k = keyw;
"";

create fhpara    = ^opara with ^paranames_, 2, 8;

/********************************************************
* RWM
********************************************************/

/** starting values **/
th1_sim    = thetaalpha[1:2];
lnpost_sim = lnpostdens( th1_sim,phi,pmean,pstdd,pshape,msel);

/*******************************************************************/

/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   if _verbose;
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);
   endif;

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim   = zeros(nsim,2);
   rej = zeros(nsim,1);
   
   /** with-in block loop **/
   j=1;
   do until j > nsim;

      /** propose **/
      th1_new = th1_sim + cc*(V_propchol*rndn(2,1));
      
      /** evaluate at the candidate **/
      lnpost_new = lnpostdens( th1_new,phi,pmean,pstdd,pshape,msel);
      
      /** accept/reject rule  **/
      r = minc(1 | exp( lnpost_new - lnpost_sim));

      if rndu(1,1) < r;

          /** accept proposed jump **/
          th1_sim    = th1_new;
          lnpost_sim = lnpost_new;

      else;

          rej[j] = 1;

      endif;
      
      parasim[j,.] = th1_sim';
      
      j = j+1;
   
   endo;               /** with-in block  **/

   if _verbose;
       " ";
       "Rejection Rate" meanc(rej);
       " ";
       "Theta 1 "; meanc(parasim[.,1:2])';
       "";
       lnpost_sim;
       "";
       
       pause(1e-20);
   endif;

   /** set finish time **/
   tfin = date;

   if _verbose;
       "Elapsed Time"; etstr(ethsec(tstart,tfin));
       "";
       "-----------------------------------------";
       "";
   endif;  /** verbose **/

   /** save output **/
   wr = writer(fhpara,parasim);

   iblock = iblock+1;

endo;  /** block **/

closeall fhpara;

/** close log file **/
output file = ^outstr off;

end;

/********************************************************
* posterior density
********************************************************/

proc (1) = lnpostdens(th1,phi,pmean,pstdd,pshape,msel);

local th1th2alp, valid, lnprior, jac, i, a, b;

if th1[2]<0;
    /* interaction parameter is out of range */
    lnprior = -1E100;
    goto eoflnpost;
endif;

/* convert parameters */
{th1th2alp, valid} = th1phi_to_th1th2alp(th1,phi);

if valid == 0;
    lnprior = -1E100;
    goto eoflnpost;
endif;
   
/* compute jacobian */
jac = dff(th1|phi);

/* evaluate prior density */
lnprior = 0;

i = 1;
do until i > rows(th1th2alp);
   if pshape[i] == 1; /* BETA Prior */
      a = (1-pmean[i])*pmean[i]^2/pstdd[i]^2 - pmean[i];
      b = a*(1/pmean[i] - 1);
      lnprior = lnprior + lpdfbeta(th1th2alp[i],a,b);   
   elseif pshape[i] == 2; /* GAMMA PRIOR */
      b = pstdd[i]^2/pmean[i];
      a = pmean[i]/b;
      lnprior = lnprior + lpdfgam(th1th2alp[i],a,b);
   elseif pshape[i] == 3; /* GAUSSIAN PRIOR */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + lpdfnor(th1th2alp[i],a,b);
   elseif pshape[i] == 4; /* INVGAMMA PRIOR */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + lpdfig(th1th2alp[i],a,b);
   elseif pshape[i] == 5; /* Uniform Prior */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + ln(1/(b-a));
   elseif pshape[i] == 6; /* Truncated Normal / symmetric around zero */
      a = pmean[i];
      b = pstdd[i];
      lnprior = lnprior + ln(2) + lpdfnor(thetaalpha[i],a,b);
   endif;
   i = i+1;
endo;

/* add Jacobian term */
lnprior = lnprior + ln(abs(det(jac)));

eoflnpost:

retp(lnprior);
endp;

/*******************************************************/

proc (2) = th1phi_to_th1th2alp(th1,phi);

local m1, m2, d1, d2, alp, valid,
      bet1, del1, bet2, del2, phi11, phi00, phi10;

valid = 1;

bet1 = th1[1];
del1 = th1[2];

phi11 = phi[1];
phi00 = phi[2];
phi10 = phi[3];

m1 = cdfn(bet1);
d1 = cdfn(bet1-del1);

m2 = 1 - phi00/(1-m1);
d2 = phi11/d1;

if (m2 < 0) OR (m2 > 1) or (d2<0) or (d2>1) or (d2>m2);
   valid = 0;
   bet2 = 0; del2 = 0; alp = 0;
   goto eoproc;
endif;

bet2 = cdfni(m2);
del2 = bet2 - cdfni(d2);

alp = ( phi10 - m1*(1-m2) - d1*(m2-d2) )/( (m1-d1)*(m2-d2) );

if (alp < 0) or (alp > 1);
   valid = 0;
   bet2 = 0; del2 = 0; alp = 0;
   goto eoproc;
endif;

eoproc:
retp( (bet1|del1|bet2|del2|alp), valid);
endp;

/********************************************/

proc (1) = th1phi_to_th1th2alpjac(th1phi);

local bet1, del1, bet2, del2, alp, phi11, phi00, phi10,
      m1, m2, d1, d2, valid;

bet1 = th1phi[1];
del1 = th1phi[2];

phi11 = th1phi[3];
phi00 = th1phi[4];
phi10 = th1phi[5];

m1 = cdfn(bet1);
d1 = cdfn(bet1-del1);

m2 = 1 - phi00/(1-m1);
d2 = phi11/d1;

if (m2 < 0) or (m2 > 1) or (d2<0) or (d2>1) or (d2>m2);
   bet2 = 0; del2 = 0; alp = 0;
   goto eoproc;
endif;

bet2 = cdfni(m2);
del2 = bet2 - cdfni(d2);

alp = ( phi10 - m1*(1-m2) - d1*(m2-d2) )/( (m1-d1)*(m2-d2) );

if (alp < 0) or (alp > 1);
   bet2 = 0; del2 = 0; alp = 0;
   goto eoproc;
endif;

eoproc:

retp(bet1|del1|bet2|del2|alp);
endp;

/********************************************/

proc (1) = dcdfni(x);
local y;
y = 1/pdfn(cdfni(x));
retp(y);
endp;

proc (1) = df1(x);
local bet1, gam1, phi11, phi00, phi10, df;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
df    = zeros(1,5);
df[1] = 1;
retp(df);
endp;

proc (1) = df2(x);
local bet1, gam1, phi11, phi00, phi10, df;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
df    = zeros(1,5);
df[2] = 1;
retp(df);
endp;

proc (1) = df3(x);
local bet1, gam1, phi11, phi00, phi10, df;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
df    = zeros(1,5);
df[1] = -dcdfni(1-phi00/(1-cdfn(bet1)))
        *phi00*pdfn(bet1)/(1-cdfn(bet1))^2;
df[4] = -dcdfni(1-phi00/(1-cdfn(bet1)))/(1-cdfn(bet1));
retp(df);
endp;

proc (1) = df4(x);
local bet1, gam1, phi11, phi00, phi10, df, dftemp;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
dftemp = df3(x);
df    = zeros(1,5);
df[1] = dftemp[1] + dcdfni(phi11/cdfn(bet1-gam1))
        *phi11*pdfn(bet1-gam1)/cdfn(bet1-gam1)^2;
df[2] = -dcdfni(phi11/cdfn(bet1-gam1))
        *phi11*pdfn(bet1-gam1)/cdfn(bet1-gam1)^2;
df[3] = -dcdfni(phi11/cdfn(bet1-gam1))
         /cdfn(bet1-gam1);
df[4] = dftemp[4];
retp(df);
endp;

proc (1) = gg(x);
local bet1, gam1, phi11, phi00, phi10, fval;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
fval = 1 - cdfn(bet1) - phi00 - phi11*(1-cdfn(bet1))/cdfn(bet1-gam1);
retp(fval);
endp;

proc (1) = AA(x);
local bet1, gam1, phi11, phi00, phi10, fval;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
fval = phi10*(1-cdfn(bet1)) - phi00*cdfn(bet1) - cdfn(bet1-gam1)*gg(x);
retp(fval);
endp;

proc (1) = BB(x);
local bet1, gam1, phi11, phi00, phi10, fval;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
fval = (cdfn(bet1) - cdfn(bet1-gam1))*gg(x);
retp(fval);
endp;


proc (1) = dgg(x);
local bet1, gam1, phi11, phi00, phi10, dg;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
dg    = zeros(1,5);
dg[1] = -pdfn(bet1) + phi11*pdfn(bet1)/cdfn(bet1-gam1)
       + phi11*(1-cdfn(bet1))*pdfn(bet1-gam1)/cdfn(bet1-gam1)^2;  
dg[2] = - phi11*(1-cdfn(bet1))*pdfn(bet1-gam1)/cdfn(bet1-gam1)^2;  
dg[3] = -(1-cdfn(bet1))/cdfn(bet1-gam1);
dg[4] = -1;
retp(dg);
endp;

proc (1) = dAA(x);
local bet1, gam1, phi11, phi00, phi10, dA, dg;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
dA    = zeros(1,5);
dg    = dgg(x);
dA[1] = -(phi10+phi00)*pdfn(bet1) - pdfn(bet1-gam1)*gg(x) - cdfn(bet1-gam1)*dg[1];
dA[2] = pdfn(bet1-gam1)*gg(x) - cdfn(bet1-gam1)*dg[2];
dA[3] = -cdfn(bet1-gam1)*dg[3];
dA[4] = -cdfn(bet1) - cdfn(bet1-gam1)*dg[4];
dA[5] = 1-cdfn(bet1) - cdfn(bet1-gam1)*dg[5];
retp(dA);
endp;

proc (1) = dBB(x);
local bet1, gam1, phi11, phi00, phi10, dB, dg;
bet1 = x[1];
gam1 = x[2];
phi11 =x[3];
phi00 =x[4];
phi10 =x[5];
dB    = zeros(1,5);
dg    = dgg(x);
dB[1] = (pdfn(bet1)-pdfn(bet1-gam1))*gg(x)
        + (cdfn(bet1) - cdfn(bet1-gam1))*dg[1];
dB[2] = pdfn(bet1-gam1)*gg(x) + (cdfn(bet1)-cdfn(bet1-gam1))*dg[2];
dB[3] = (cdfn(bet1)-cdfn(bet1-gam1))*dg[3];
dB[4] = (cdfn(bet1)-cdfn(bet1-gam1))*dg[4];
retp(dB);
endp;

proc (1) = df5(x);
local dA, dB, df;
dA = dAA(x);
dB = dBB(x);
df = (dA*BB(x)-AA(x)*dB)/BB(x)^2;
retp(df);
endp;

proc (1) = dff(x);
local df;
df = df1(x) | df2(x) | df3(x) | df4(x)| df5(x);
retp(df);
endp;
